// WinAVR 20030913
// Target : 8535
// Crystal: 8.0000Mhz

#include <avr/io.h>

#include "t6963.h"
//#include "uart.h"

void t6963_status_chk(u08 status)
{
	u08 tmp;
	
	DATA_DIR_IN();
	CTRL();
	
	do
	{
		RD_LOW();
		CE_LOW();
		t6963_wait_150_ns();
		tmp = DATA_LINES_PIN;
		CE_HIGH();
		RD_HIGH();
	} while((tmp & status) != status);
	
	DATA_DIR_OUT();
}

void t6963_write(u08 data)	// put a byte on the data lines ...
{
	WR_LOW();
	DATA_LINES_PORT = data;
	CE_LOW();
	t6963_wait_80_ns();
	CE_HIGH();
	WR_HIGH();
}

void t6963_write_cmd(u08 cmd)	// write a command without data
{
	t6963_status_chk(STATUS_READ_WRITE);
	CTRL();
	t6963_write(cmd);
}

void t6963_write_cmd1(u08 cmd, u08 data1)	// write a command with one data byte
{
	t6963_status_chk(STATUS_READ_WRITE);
	DATA();
	t6963_write(data1);
	t6963_status_chk(STATUS_READ_WRITE);
	CTRL();
	t6963_write(cmd);
}

void t6963_write_cmd2(u08 cmd, u08 data1, u08 data2)	// write a command with two data byte
{
	t6963_status_chk(STATUS_READ_WRITE);
	DATA();
	t6963_write(data1);
	t6963_status_chk(STATUS_READ_WRITE);
	DATA();
	t6963_write(data2);
	t6963_status_chk(STATUS_READ_WRITE);
	CTRL();
	t6963_write(cmd);
}

void t6963_clear_ram(u16 adr, u16 len)
{
	t6963_write_cmd2(CMD_SET_ADRESS_POINTER , lo8(adr), hi8(adr));
	t6963_write_cmd(CMD_SET_AUTO_WRITE);
	
	while(len)
	{
		t6963_status_chk(STATUS_AUTO_WRITE);
		DATA();
		t6963_write(0);
		len--;
	}
	
	t6963_status_chk(STATUS_AUTO_WRITE);
	t6963_write_cmd(CMD_AUTO_RESET);
}

void t6963_init(void)
{
	sbi(CTRL_LINES_DDR, CTRL_FS);
	sbi(CTRL_LINES_DDR, CTRL_CE);
	sbi(CTRL_LINES_DDR, CTRL_CD);
	sbi(CTRL_LINES_DDR, CTRL_WR);
	sbi(CTRL_LINES_DDR, CTRL_RD);		// outputs

	#ifdef USE_8X8_FONT
		FONT_8X8();
	#else
		FONT_6X8();
	#endif

	CE_HIGH();
	WR_HIGH();
	RD_HIGH();
	
	DATA_DIR_OUT();

	// Sleep 100 ms
	
	t6963_write_cmd2(CMD_SET_CURSOR_POINTER, 0, 0);
	
	t6963_write_cmd2(CMD_SET_OFFSET_REGISTER, hi8(CG_RAM_OFFSET), 0);
	
	t6963_write_cmd2(CMD_SET_ADRESS_POINTER , 0, 0);

	t6963_write_cmd2(CMD_SET_TEXT_HOME_ADR, lo8(TEXT_START), hi8(TEXT_START));
	
	t6963_write_cmd2(CMD_SET_TEXT_AREA, TEXT_BYTES_PER_LINE, 0);
	
	t6963_write_cmd2(CMD_SET_GRAPHIC_HOME_ADR, lo8(G_START), hi8(G_START));
	
	t6963_write_cmd2(CMD_SET_GRAPHIC_AREA, G_BYTES_PER_LINE, 0);

	t6963_write_cmd(CMD_MODE_SET);
	
	t6963_write_cmd(CMD_DISPLAY_MODE);
	
	t6963_write_cmd(CMD_CUR_PATTERN_SEL);
	
	t6963_clear_ram(TEXT_START, TEXT_SREEN_SIZE);
	t6963_clear_ram(G_START, G_SREEN_SIZE);
}

void t6963_copy_bitmap(u16 lcd_adr, PGM_P bitmap, u08 cols, u08 rows, u08 invert)
{
	// lcd_adr = start adress in the display RAM
	// bitmap = pointer to the bitmap in the flash
	// cols = cols of the bitmap in bytes (pixel / 8)
	// rows = rows of the bitmap in pixel
	// invert = if > 0, then bitmap is inverted (maybe a bug in BMP2C)
	
	u08 row, col, data;
	u16 lcd_ptr;

	for(row=0; row < rows; row++)
	{
		lcd_ptr = lcd_adr + (row * cols);
		t6963_write_cmd2(CMD_SET_ADRESS_POINTER , lo8(lcd_ptr), hi8(lcd_ptr));

		for(col=0; col < cols; col++)
		{
			data = pgm_read_byte(bitmap + (row * cols) + col);
			if(invert) data = ~data;
			t6963_write_cmd1(CMD_DATA_WR_INC, data);
		}
	}
}
