// WinAVR 20030913
// Target : 8535
// Crystal: 8.0000Mhz

#if !defined(T6963_INCLUDED)
#define T6963_INCLUDED

#include <avr/pgmspace.h>
#include "dat_types.h"


// AMPIRE AG-12864DYJQY-00H  - Display
// ------------------------------------
// Size: 128 x 64 pixel
// RAM:  32kByte (W24257S)
#define LCD_WIDTH							128
#define LCD_HEIGHT						64
#define LCD_RAM_SIZE					32768			// 0x0000..0x7FFF
#define USE_8X8_FONT										// If you would like 6x8 font, comment this line ...

#define TEXT_BYTES_PER_LINE		16				// LCD_WIDTH / FONT_SIZE ...
#define G_BYTES_PER_LINE			16				// LCD_WIDTH / 8

#define TEXT_SREEN_SIZE				TEXT_BYTES_PER_LINE * (LCD_HEIGHT / 8)	//  128 Byte
#define G_SREEN_SIZE					G_BYTES_PER_LINE * LCD_HEIGHT						// 1024 Byte

#define TEXT_START						0x0000
#define TEXT_END							0x13FF		// 40 text screens

#define G_START								0x1400
#define G_END									0x77FF		// 25 graphic screens

#define	CG_RAM_START					0x7800
#define	CG_RAM_END						0x7FFF		// CG-RAM must be 2048 Byte !
#define CG_RAM_OFFSET					(CG_RAM_START & 0xF800)	// upper 5 bits of the start address

#define DATA_LINES_PORT				PORTC
#define DATA_LINES_PIN				PINC
#define DATA_LINES_DDR				DDRC
#define CTRL_LINES_PORT				PORTA
#define CTRL_LINES_PIN				PINA
#define CTRL_LINES_DDR				DDRA
#define CTRL_FS								PA4
#define CTRL_CE								PA3
#define CTRL_CD								PA2
#define CTRL_WR								PA1
#define CTRL_RD								PA0

/*************************************************************************/
#define	FONT_6X8()						sbi(CTRL_LINES_PORT, CTRL_FS)
#define	FONT_8X8()						cbi(CTRL_LINES_PORT, CTRL_FS)

#define	CE_HIGH()							sbi(CTRL_LINES_PORT, CTRL_CE)
#define	CE_LOW()							cbi(CTRL_LINES_PORT, CTRL_CE)

#define	CTRL()								sbi(CTRL_LINES_PORT, CTRL_CD)
#define	DATA()								cbi(CTRL_LINES_PORT, CTRL_CD)

#define	WR_HIGH()							sbi(CTRL_LINES_PORT, CTRL_WR)
#define	WR_LOW()							cbi(CTRL_LINES_PORT, CTRL_WR)

#define	RD_HIGH()							sbi(CTRL_LINES_PORT, CTRL_RD)
#define	RD_LOW()							cbi(CTRL_LINES_PORT, CTRL_RD)

#define DATA_DIR_IN()					DATA_LINES_DDR=0x00	// set IO-pind to inputs
#define DATA_DIR_OUT()				DATA_LINES_DDR=0xFF	// set IO-pind to outputs
/*************************************************************************/
#define STATUS_READ_WRITE			0x3		// STA0 + STA1 bit must be high
#define STATUS_AUTO_READ			0x4		// 				STA2 bit must be high
#define STATUS_AUTO_WRITE			0x8		// 				STA3 bit must be high
/*************************************************************************/
#define CMD_SET_CURSOR_POINTER		0x21
#define CMD_SET_OFFSET_REGISTER		0x22
#define CMD_SET_ADRESS_POINTER		0x24

#define CMD_SET_TEXT_HOME_ADR			0x40
#define CMD_SET_TEXT_AREA					0x41
#define CMD_SET_GRAPHIC_HOME_ADR	0x42
#define CMD_SET_GRAPHIC_AREA			0x43

#define CMD_SET_AUTO_WRITE				0xB0
#define CMD_SET_AUTO_READ					0xB1
#define CMD_AUTO_RESET						0xB2

#define CMD_DATA_WR_INC						0xC0

#define CMD_MODE_SET							0x80	// OR mode, int. CG-ROM MODE						adjust to your needs !
#define CMD_DISPLAY_MODE					0x9F	// text on, graphic on , cursor blink		adjust to your needs !
#define CMD_CUR_PATTERN_SEL				0xA0	// 1-line cursor												adjust to your needs !



#define t6963_wait_150_ns()				asm volatile( "nop\n\t" "nop\n\t");
#define t6963_wait_80_ns()				asm volatile( "nop\n\t");

#define lo8(val)	 val & 0x00FF
#define hi8(val)	(val & 0xFF00) >> 8

void t6963_status_chk(u08 status);

void t6963_write(u08 data);														// put a byte on the data lines ...
void t6963_write_cmd(u08 cmd);												// write a command without data
void t6963_write_cmd1(u08 cmd, u08 data1);						// write a command with one data byte
void t6963_write_cmd2(u08 cmd, u08 data1, u08 data2);	// write a command with two data byte

void t6963_clear_ram(u16 adr, u16 len);

void t6963_init(void);
void t6963_copy_bitmap(u16 lcd_adr, PGM_P bitmap, u08 cols, u08 rows, u08 invert);


#endif // !defined(T6963_INCLUDED)
